/*
 * $Id$
 *
 * Copyright (C) 2009, 2010 Elias Norberg, Patrik Sternudd
 *
 * This file is part of TDEPS.
 *
 * TDEPS is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TDEOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TDEPS.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <gnutls/gnutls.h>
#include "comm.h"

#define CAFILE "ca-cert.pem"
#define MSG "GET / HTTP/1.0\r\n\r\n\r\n"

int
main ( int argc, char *argv[] )
{
	int ret, sd, ii;
	gnutls_session_t session;
	char buffer[1024];
	const char *err;
	gnutls_certificate_credentials_t xcred;

	if( argc != 3 ){
		printf("Usage: %s <server> <port>!\n", argv[0]);
		return 0;
	}

	gnutls_global_init();

	gnutls_certificate_allocate_credentials(&xcred);

	ret = gnutls_certificate_set_x509_trust_file(xcred, CAFILE, GNUTLS_X509_FMT_PEM);
	if( ret < 0 ){
		fprintf(stderr, "Cannot set certification: ");
		gnutls_perror(ret);
		return -1;
	}

	gnutls_init(&session, GNUTLS_CLIENT);

	ret = gnutls_priority_set_direct( session, "PERFORMANCE", &err );
	if( ret < 0 ){
		if( ret == GNUTLS_E_INVALID_REQUEST ){
			fprintf(stderr, "Syntax error at %s\n", err);
		}
		return 1;
	}

	gnutls_credentials_set( session, GNUTLS_CRD_CERTIFICATE, xcred );

	sd = connection_tcp_connect( argv[1], atoi(argv[2]));
	if( sd == -1 ){
		perror("connection_tcp_connect()");
		return -1;
	}

	gnutls_transport_set_ptr ( session, (gnutls_transport_ptr_t)sd );

	ret = gnutls_handshake( session );

	if( ret < 0 ){
		fprintf(stderr, "Handshake failed\n");
		gnutls_perror(ret);
		return -1;
	}
	printf("Handshake completed\n");

	connection_print_session_info(session);
	
	gnutls_record_send(session, "ABC DEF GEH", 11);

	ret = gnutls_record_recv(session, buffer, 1024);
	if( ret == 0 ){
		printf("Peer has closed the TLS connection\n");
		return -1;
	}else if( ret < 0 ){
		fprintf(stderr, "Error: %s\n", gnutls_strerror(ret));
		return -1;
	}

	printf("Recieved %d bytes: ", ret );
	for(ii=0;ii<ret;ii++)
		fputc(buffer[ii],stdout);
	fputc('\n',stdout);

	gnutls_bye(session, GNUTLS_SHUT_RDWR);

	close(sd);
	gnutls_deinit(session);
	gnutls_certificate_free_credentials(xcred);
	gnutls_global_deinit();

	return 0;
}
