/*
 * $Id$
 *
 * Copyright (C) 2009, 2010 Elias Norberg, Patrik Sternudd
 *
 * This file is part of TDEPS.
 *
 * TDEPS is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TDEOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TDEPS.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <iostream>
#include <stdexcept>
#include <gnutls/gnutls.h>
#include <gnutls/gnutlsxx.h>
#include <cstring>
#include "communication.h"

#define KEYFILE "key.pem"
#define CERTFILE "cert.pem"
#define CAFILE "ca-cert.pem"
#define CRLFILE "crl.pem"

#if 0
class comm{
	private:
		gnutls::certificate_credentials x509_cred;
		gnutls::priority priority_cache;
		gnutls::dh_params dh_params;

		char *trust_file = NULL;
		char *crl_file = NULL;
		char *key_file = NULL;
		char *cert_file = NULL;

		/* Bits to use for DH-params */
		int dh_params_bits = 1024;
	public:
		int initialize_tls();
		int generate_dh_params();
		int print_session_info(gnutl_session_t session);
		int tls_session_set_credentials(gnutls::server_session &sess);

		int set_trust_file(char *filename){ this->trust_file = filename; return 0; }
		int set_crl_file(char *filename){ this->crl_file = filename; return 0; }
		int set_key_file(char *filename){ this->key_file = filename; return 0; }
		int set_cert_file(char *filename){ this->cert_file = filename; return 0; }
		int set_dh_bits(int bits){ this->dh_params_bits = bits; return 0; }

		char *get_trust_file(){ return this->trust_file; }
		char *get_crl_file(){ return this->crl_file; }
		char *get_key_file(){ return this->key_file; }
		char *get_cert_file(){ return this->cert_file; }
		char *get_dh_bits(){ return this->dh_params_bits; }
}
#endif

/* 
 * comm::initialize_tls()
 * Initialize TLS using the data set previously
 */
int comm::initialize_tls()
{
	/* FIXME: set error */
	if( this->trust_file == NULL || this->crl_file == NULL || this->key_file == NULL || this->cert_file == NULL )
		return -1;

	try{
		this->x509_cred.set_x509_trust_file(this->trust_file, GNUTLS_X509_FMT_PEM);
		this->x509_cred.set_x509_crl_file(this->crl_file, GNUTLS_X509_FMT_PEM);
		this->x509_cred.set_x509_key_file(this->cert_file, this->key_file, GNUTLS_X509_FMT_PEM);
	}catch(std::exception &ex){
		std::cout << "Error setting up x509 cred: " << ex.what() << std::endl;
		throw ex;
	}

	std::cout << "Generating DH Parameters. Please wait...";
	fflush(stdout);
	dh_params.generate(this->dh_params_bits);
	std::cout << "done" << std::endl;

	/*
	ret = gnutls_priority_init(&priority_cache, "NORMAL", &err);
	if( ret < 0 ){
		if( ret == GNUTLS_E_INVALID_REQUEST ){
			fprintf(stderr, "Syntax error at %s\n", err);
		}
		return 1;
	}
	*/

	this->x509_cred.set_dh_params(this->dh_params);

	return 0;
}


int comm::tls_session_set_credentials(gnutls::server_session *sess)
{

	try{
		sess->set_credentials(this->x509_cred);
		sess->set_certificate_request(GNUTLS_CERT_REQUEST);
	}catch(std::exception &ex){
		return -1;
	}
	return 0;
}

comm::comm()
{
	/* Set default values */
	this->dh_params_bits = 1024;
	this->trust_file = NULL;
	this->crl_file = NULL;
	this->key_file = NULL;
	this->cert_file =NULL;

}
