/*
 * $Id$
 *
 * Copyright (C) 2009, 2010 Elias Norberg, Patrik Sternudd
 *
 * This file is part of TDEPS.
 *
 * TDEPS is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TDEOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TDEPS.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <iostream>
#include <fstream>
#include <string>
#include <cstdlib>
#include <string.h>
#include "config.h"
#include "linked_list.h"

/*
class config{
	private:
		//linked_list::list *list;

	public:
		config( const char *filename );
		config(){ };
		~config(){ delete this->list };
		bool load( const char *filename );
		bool load( std::string filename );
		
		int get( const char *setting );
		char *get( const char *setting );
		std::string *get( const char *setting );
		bool get( const char *settings );
		
		bool set( const char *setting, int value );
		bool set( const char *setting, char *value );
		bool set( const char *setting, std::string value );
		bool set( const char *setting, bool value );
};
*/

config::config( const char *filename )
{
	this->load(filename);
}

bool config::load( const char *filename )
{
	std::ifstream fd;
	std::string line, setting;

	fd.open( filename, std::ios::in );
	
	/* If we can't open the config-file,
		don't bother setting values
	 */
	if( ! fd )
		return false;

	while( ! fd.eof() ){
		unsigned int pos;

		std::getline( fd, line );

		/* Remove spaces at the beginning */
		for( pos = 0; (line[pos] == ' ' || line[pos] == '\t') && pos < line.length() ; pos ++ );
		if( pos ) line.erase(0, pos);

		if( line.length() <= 0 )
			continue;

		/* Skip comments */
		if( line[0] == '#' ) continue;
		
		for( pos = 0; (line[pos] != ' ' && line[pos] != '\t' ) && pos < line.length() ; pos ++ );
		setting = line.substr(0,pos);

		/* Remove everything except value */
		pos ++;	
		for(; (line[pos] == ' ' || line[pos] == '\t') && pos < line.length() ; pos ++ );
		line.erase(0, pos);

		/* Remove trailing spaces */
		for( pos = line.length() - 1; 
				(line[pos] == '\n' ||
				 line[pos] == '\r' ||
				 line[pos] == ' ' ||
				 line[pos] == '\t' ) && pos > 0 ; pos -- );
		if( pos < line.length() - 1 ) line.erase(pos + 1, line.length() );

		this->set( setting.c_str(), line );
	}

	fd.close();

	return true;
}

bool config::set( const char *setting, std::string value )
{

	if( this->settings == NULL )
		this->settings = new linked_list::list;
	
	this->settings->add(setting, value.c_str() );
	return true;
}

bool config::get_bool( const char *setting )
{
	const char *tmp = settings->get(setting);

	if( ! tmp ) return false;
	if( tmp[0] == '0' && tmp[1] == '\0' ) return false;
	if( strcasecmp(tmp, "false") ) return false;

	return true;
}

int config::get_int( const char *setting )
{
	const char *tmp = settings->get(setting);

	if( ! tmp ) return -1;
	return atoi(tmp);
}

char *config::get_str( const char *setting )
{
	return settings->get(setting);
}

std::string *config::get_string( const char *setting )
{
	return new std::string(settings->get(setting));
}
