/*
 * $Id$
 *
 * Copyright (C) 2009, 2010 Elias Norberg, Patrik Sternudd
 *
 * This file is part of TDEPS.
 *
 * TDEPS is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TDEOS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TDEPS.  If not, see <http://www.gnu.org/licenses/>.
 */
#include <iostream>
#include <stdexcept>
#include <gnutls/gnutls.h>
#include <gnutls/gnutlsxx.h>
#include <cstring>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <stdlib.h>
#include <gcrypt.h>
#include "communication.h"
#include "config.h"

using namespace std;

#define KEYFILE "key.pem"
#define CERTFILE "cert.pem"
#define CAFILE "ca-cert.pem"
#define CRLFILE "crl.pem"

int main( int argc, char *argv[])
{
	gnutls::server_session *session;
	int ret, sd;
	char buffer[1024];
	int listen_sd;
	struct sockaddr_in sa_serv;
	struct sockaddr_in sa_cli;
	socklen_t client_len;
	int server_port;
	comm *c;
	config *conf;

	conf = new config("tdeps-server.conf");

	cout << "ServerMode: " << conf->get_str("ServerMode") << endl;
	server_port = conf->get_int("ServerPort");

	if( server_port <= 0 || server_port > 65535 ){
		cout << "Invalid server port specified (" << server_port << 
			"). Falling back to standard port (" << DEFAULT_SERVER_PORT << ")" << endl;
		server_port = DEFAULT_SERVER_PORT;
	}

//	gcry_control(GCRYPCTL_ENABLE_QUICK_RANDOM, 0);
	gnutls_global_init();

	c = new comm();

	c->set_trust_file((char*)CAFILE);
	c->set_crl_file((char*)CRLFILE);
	c->set_cert_file((char*)CERTFILE);
	c->set_key_file((char*)KEYFILE);
	c->initialize_tls();
	
	if( (listen_sd = socket(AF_INET, SOCK_STREAM, 0)) == -1 ){
		perror("socket()");
		return -1;
	}

	memset(&sa_serv, 0, sizeof(sa_serv));
	sa_serv.sin_family = AF_INET;
	sa_serv.sin_addr.s_addr = INADDR_ANY;
	sa_serv.sin_port = htons(server_port);

	/*setsockopt(listen_sd, SOL_SOCKET, SO_REUSEADDR, (void*)&optval, sizeof(int));
	*/

	if( bind(listen_sd, (struct sockaddr*)&sa_serv, sizeof(sa_serv)) == -1 ){
		perror("bind()");
		return -1;
	}

	if( listen(listen_sd, 1024) == -1 ){
		perror("listen()");
		return -1;
	}

	printf("Server ready on port %d\n", server_port);

	client_len = sizeof(sa_cli);
	for(;;){
		session = new gnutls::server_session();
		c->tls_session_set_credentials(session);
		session->set_priority("NORMAL", NULL);

		sd = accept(listen_sd, (struct sockaddr*)&sa_cli, &client_len);
		if( sd == -1 ){
			perror("accept()");
			break;
		}

		printf("Connection from %s:%d\n",
			inet_ntop(AF_INET,&sa_cli.sin_addr, buffer,
				sizeof(buffer)), ntohs(sa_cli.sin_port));

		try{
			session->set_transport_ptr((gnutls_transport_ptr_t)sd);
			session->handshake();
		}catch(std::exception &ex){
			std::cout << "Handshake has failed: " << ex.what() << std::endl;
			delete session;
			continue;
		}

		printf("Handshake complete\n");

		for(;;){
			memset(buffer, 0, sizeof(buffer));
			ret = session->recv(buffer, sizeof(buffer));

			if( ret == 0 ){
				printf("Client has closed the GNUTLS connection\n");
				break;
			}else if( ret < 0 ){
				fprintf(stderr, "Recieved corrupted data(%d): %s\n",
					ret, gnutls_strerror(ret));
				break;
			}

			session->send(buffer, strlen(buffer));
		}

		session->bye(GNUTLS_SHUT_WR);

		close(sd);
		delete session;
	}

	close(listen_sd);


	gnutls_global_deinit();

	delete conf;
	return 0;
}
